package com.huimv.farm.damsubsidy.controller;

import cn.hutool.core.date.DateTime;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.entity.BaseVeterStation;
import com.huimv.farm.damsubsidy.service.IBaseVeterStationService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author author
 * @since 2023-05-05
 */
@CrossOrigin
@RestController
@RequestMapping("/base-veter-station")
public class BaseVeterStationController {

    @Autowired
    private IBaseVeterStationService baseVeterStationService;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    public Result addMenu(@RequestBody BaseVeterStation baseVeterStation , HttpServletRequest request){
        baseVeterStation.setCreateUser(TokenSign.getMemberIdByJwtToken(request));
        baseVeterStation.setCreateTime(DateTime.now());
        baseVeterStationService.save(baseVeterStation);
        updateCache();
        return new Result(ResultCode.SUCCESS,"添加成功");
    }

    @PostMapping("/edit")
    @Transactional(rollbackFor = Exception.class)
    public Result editMenu(@RequestBody BaseVeterStation baseVeterStation, HttpServletRequest httpServletRequest){
        baseVeterStation.setUpdateUser(TokenSign.getUserName(httpServletRequest));
        baseVeterStation.setUpdateTime(DateTime.now());
        baseVeterStationService.updateById(baseVeterStation);
        updateCache();
        return new Result(ResultCode.SUCCESS,"修改成功");
    }

    @RequestMapping("/remove")
    @Transactional(rollbackFor = Exception.class)
    public Result removeMenu(@RequestParam(name = "id") Integer id){
        baseVeterStationService.removeById(id);
        updateCache();
        return new Result(ResultCode.SUCCESS,"删除成功");
    }

    @PostMapping("/list")
    @Transactional(rollbackFor = Exception.class)
    public Result list() {
        String veterStation = redisTemplate.opsForValue().get("veterStation");
        List<BaseVeterStation> baseVeterStationList = null;
        if (StringUtils.isEmpty(veterStation)){
            System.out.println("-----------获取兽医站数据库数据-----------");
            baseVeterStationList = baseVeterStationService.list();
            String veterStations = JSON.toJSON(baseVeterStationList).toString();
            redisTemplate.opsForValue().set("veterStation",veterStations);
        }else {
            System.out.println("-----------获取兽医站缓存数据-----------");
            baseVeterStationList = JSONObject.parseArray(veterStation,BaseVeterStation.class);
        }
        return new Result(ResultCode.SUCCESS, baseVeterStationList);
    }

    @PostMapping("/page")
    public Result page( @RequestBody Map map) {
        return baseVeterStationService.page(map);
    }

    private void updateCache(){
        List<BaseVeterStation> baseVeterStationList = baseVeterStationService.list();
        String veterStations = JSON.toJSON(baseVeterStationList).toString();
        try {
            redisTemplate.opsForValue().set("veterStation",veterStations);
            System.out.println("兽医站缓存更新成功");
        } catch (Exception e) {
            System.out.println("兽医站数据缓存失败");
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            e.printStackTrace();
        }
    }
}
